@@ -11,10 +11,10 @@ use crate::tokenizer::Tokenizer;
 use crate::Result;
 
 /// Represents the default `Evaluator`.
-#[derive(Clone)]
 pub struct Evaluator<'a, N, C: Context<'a, N> = DefaultContext<'a, N>> {
     /// The context used for evaluation.
     context: C,
+    tokenizer: Tokenizer<'a, N, C>,
     _marker: &'a PhantomData<N>,
 }
 
@@ -30,6 +30,7 @@ impl<'a, N: CheckedNum> Evaluator<'a, N, DefaultContext<'a, N>> {
     pub fn new() -> Self {
         Evaluator {
             context: DefaultContext::new_checked(),
+            tokenizer: Tokenizer::new(),
             _marker: &PhantomData,
         }
     }
@@ -44,6 +45,7 @@ where
     pub fn with_context(context: C) -> Self {
         Evaluator {
             context,
+            tokenizer: Tokenizer::new(),
             _marker: &PhantomData,
         }
     }
@@ -61,6 +63,19 @@ where
     }
 }
 
+impl<'a, N, C> Evaluator<'a, N, C>
+where
+    C: Context<'a, N>,
+{
+    pub fn with_context_and_tokenizer(context: C, tokenizer: Tokenizer<'a, N, C>) -> Self {
+        Evaluator {
+            context,
+            tokenizer,
+            _marker: &PhantomData,
+        }
+    }
+}
+
 impl<'a, N, C> Evaluator<'a, N, C>
 where
     C: Context<'a, N>,
@@ -84,8 +99,7 @@ where
     #[inline]
     pub fn eval(&'a self, expression: &str) -> Result<N> {
         let context = self.context();
-        let tokenizer = Tokenizer::with_context(context);
-        let tokens = tokenizer.tokenize(expression)?;
+        let tokens = self.tokenizer.tokenize(context, expression)?;
         self.eval_tokens(&tokens)
     }
 }
@@ -956,10 +970,10 @@ mod tests {
         let context = DefaultContext::with_config_checked(config);
         let mut evaluator: Evaluator<i64> = Evaluator::with_context(context);
 
-        evaluator.mut_context().set_variable("x", 10);
+        evaluator.mut_context().set_variable("x", 10).unwrap();
         assert_eq!(evaluator.eval("2x").unwrap(), 20);
 
-        evaluator.mut_context().set_variable("x", 5);
+        evaluator.mut_context().set_variable("x", 5).unwrap();
         assert_eq!(evaluator.eval("3x").unwrap(), 15);
 
         assert!(evaluator.eval("2Sin(50)").is_ok());
@@ -1026,7 +1040,7 @@ mod tests {
     #[test]
     fn eval_using_variable_test() {
         let mut evaluator = Evaluator::new();
-        evaluator.mut_context().set_variable("x", 10);
+        evaluator.mut_context().set_variable("x", 10).unwrap();
 
         assert_eq!(evaluator.eval("x + 2").unwrap(), 12);
     }
@@ -1049,7 +1063,7 @@ mod tests {
         }
 
         let mut context: DefaultContext<f64> = DefaultContext::new();
-        context.add_function(SumFunction);
+        context.add_function(SumFunction).unwrap();
 
         let evaluator = Evaluator::with_context(context);
         assert_eq!(evaluator.eval("sum(1, 2, 3, 4)").unwrap(), 10.0);
