@@ -1,6 +1,7 @@
 use std::cmp::Ordering;
 use std::fmt::{Debug, Display, Formatter};
 use std::hash::{Hash, Hasher};
+use std::ops::Deref;
 
 /// Represents `str` that will ignore case when comparing.
 #[derive(Copy, Clone)]
@@ -15,7 +16,7 @@ impl<'a> IgnoreCaseStr<'a> {
 
     /// Gets a reference to the inner value.
     #[inline]
-    pub fn as_raw_str(&self) -> &'a str {
+    pub fn as_str(&self) -> &'a str {
         self.0
     }
 
@@ -26,6 +27,15 @@ impl<'a> IgnoreCaseStr<'a> {
     }
 }
 
+impl Deref for IgnoreCaseStr<'_> {
+    type Target = str;
+
+    #[inline]
+    fn deref(&self) -> &str {
+        self.0
+    }
+}
+
 impl<'a> From<&'a str> for IgnoreCaseStr<'a> {
     #[inline]
     fn from(value: &'a str) -> Self {
